# Copyright (c) 2009-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: addform.py 41849 2010-05-07 12:00:45Z sylvain $

import martian

from zope.component import provideUtility
from zope.component.interfaces import IFactory

from zeam.form.silva.form import SMIAddForm
from silva.core import conf as silvaconf


class AddFormGrokker(martian.ClassGrokker):
    """ Grok add form and register them as factories.
    """
    martian.component(SMIAddForm)
    martian.directive(silvaconf.name)

    def execute(self, form, name, config, **kw):
        config.action(
            discriminator = ('utility', IFactory, name),
            callable = provideUtility,
            args = (form, IFactory, name))
        return True



# this is da package man

from zeam.form.silva.actions import *
from zeam.form.composed import *
from zeam.form.base import *
from zeam.form.ztk import *
from zeam.form.table import *

from zeam.form.silva.form import ZMIForm, ZMIComposedForm, ZMISubForm
from zeam.form.silva.form import SMIForm, SMIAddForm, SMIEditForm
from zeam.form.silva.form import SMIComposedForm, SMISubForm, SMISubFormGroup
from zeam.form.silva.form import SMISubTableForm
from zeam.form.silva.form import SMIViewletForm, PublicViewletForm
from zeam.form.silva.form import SMIContentProviderForm, PublicContentProviderForm
from zeam.form.silva.form import SilvaDataManager
from zeam.form.silva.form import PublicForm
from zeam.form.silva.rest import RESTPopupForm, RESTKupuEditProperties
from zeam.form.silva.actions import EditAction, CancelAction, PopupAction



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2011 Infrae. All rights reserved.
# See also LICENSE.txt

from datetime import datetime
from DateTime import DateTime

from ZPublisher.HTTPRequest import FileUpload
from Products.Silva.icon import get_icon_url

from five import grok
from zope.interface import Interface

from silva.core.smi.interfaces import ISMILayer
from silva.core.conf import schema as silvaschema
from silva.translations import translate as _

from zeam.form.base.interfaces import IWidget, IWidgetExtractor
from zeam.form.base.markers import NO_VALUE
from zeam.form.base.widgets import WidgetExtractor
from zeam.form.ztk.fields import SchemaField, SchemaFieldWidget
from zeam.form.ztk.fields import registerSchemaField
from zeam.form.ztk.interfaces import ICollectionSchemaField
from zeam.form.ztk.widgets.collection import newCollectionWidgetFactory
from zeam.form.ztk.widgets.date import DatetimeSchemaField
from zeam.form.ztk.widgets.textline import TextLineSchemaField

def register():
    registerSchemaField(FileSchemaField, silvaschema.IBytes)


class FileSchemaField(SchemaField):
    """Field to upload a file.
    """


class FileWidgetInput(SchemaFieldWidget):
    grok.adapts(FileSchemaField, Interface, Interface)
    grok.name('input')

    def valueToUnicode(self, value):
        return u''


class FileWidgetExtractor(WidgetExtractor):
    grok.adapts(FileSchemaField, Interface, Interface)

    def extract(self):
        value = self.request.form.get(self.identifier, u'')
        if not isinstance(value, FileUpload) or not value:
            value = NO_VALUE
        return value, None


class DateTimeFieldWidget(SchemaFieldWidget):
    grok.adapts(DatetimeSchemaField, Interface, ISMILayer)

    def prepareContentValue(self, value):
        if value is NO_VALUE:
            return {self.identifier + '.year': u'',
                    self.identifier + '.month': u'',
                    self.identifier + '.day': u'',
                    self.identifier + '.hour': u'',
                    self.identifier + '.min': u'',
                    self.identifier: u''}
        if isinstance(value, DateTime):
            value = value.asdatetime()
        return {self.identifier + '.year': u'%d' % value.year,
                self.identifier + '.month': u'%02d' % value.month,
                self.identifier + '.day': u'%02d' % value.day,
                self.identifier + '.hour': u'%02d' % value.hour,
                self.identifier + '.min': u'%02d' % value.minute,
                self.identifier: u''}


class DateTimeWidgetExtractor(WidgetExtractor):
    grok.adapts(DatetimeSchemaField, Interface, ISMILayer)

    def extract(self):
        identifier = self.identifier
        value = self.request.form.get(identifier, None)
        if value is None:
            return NO_VALUE, None

        def extract(key, min_value=None, max_value=None, required=True):
            value = self.request.form.get('.'.join((identifier, key)), None)
            if not value:
                if required:
                    raise ValueError(u'Missing %s value' %key)
                return min_value
            try:
                value = int(value)
            except ValueError:
                raise ValueError((u'%s is not a number' % key).capitalize())
            if min_value and max_value:
                if value < min_value or value > max_value:
                    raise ValueError((u'%s is not within %d and %d' % (
                                key, min_value, max_value)).capitalize())
            return value
        try:
            year = extract('year', None, None, self.component.required)
            day = extract('day', 1, 31, year is not None)
            month = extract('month', 1, 12, year is not None)
            hour = extract('hour', 0, 23, False)
            minute = extract('min', 0, 59, False)
        except ValueError, error:
            return (None, _(error))
        if year is None:
            return (NO_VALUE, None)
        try:
            return (datetime(year, month, day, hour, minute), None)
        except ValueError, error:
            return (None, _(str(error).capitalize()))


class IconDisplayWidget(SchemaFieldWidget):
    grok.name('silva.icon')

    def update(self):
        super(IconDisplayWidget, self).update()
        content = self.form.getContentData().getContent()
        self.icon_url = get_icon_url(content, self.request)


grok.global_adapter(
    newCollectionWidgetFactory(mode='lines'),
    adapts=(ICollectionSchemaField, Interface, Interface),
    provides=IWidget,
    name='lines')


grok.global_adapter(
    newCollectionWidgetFactory(mode='lines', interface=IWidgetExtractor),
    adapts=(ICollectionSchemaField, Interface, Interface),
    provides=IWidgetExtractor,
    name='lines')


class LinesWidget(SchemaFieldWidget):
    grok.adapts(ICollectionSchemaField, TextLineSchemaField, Interface, Interface)
    grok.name('lines')

    def __init__(self, field, value_field,form, request):
        super(LinesWidget, self).__init__(field, form, request)
        self.text_component = value_field

    def valueToUnicode(self, value):
        return u'\n'.join(value)


class LinesWidgetExtractor(WidgetExtractor):
    grok.adapts(ICollectionSchemaField, TextLineSchemaField, Interface, Interface)
    grok.name('lines')

    def __init__(self, field, value_field,form, request):
        super(LinesWidgetExtractor, self).__init__(field, form, request)
        self.text_component = value_field

    def extract(self):
        value, errors = super(LinesWidgetExtractor, self).extract()
        if errors is None:
            if value is not NO_VALUE:
                value = self.component.collectionType(
                    filter(lambda v: v,
                           map(lambda s: s.strip('\r'),
                               value.split('\n'))))
        return (value, errors)



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from five import grok
from infrae import rest
from megrok import pagetemplate as pt
from silva.core.interfaces import IVersionedContent
from silva.core.smi.interfaces import ISMILayer
from zope.i18n import translate
from zope.interface import alsoProvides

from zeam.form.base.form import FormCanvas
from zeam.form.base.interfaces import IFormCanvas
from zeam.form.base.markers import SUCCESS
from zeam.form.base.widgets import getWidgetExtractor
from zeam.form.silva import interfaces
from zeam.form.silva.form import SilvaDataManager
from zeam.form.silva.form import SilvaFormData, SMIComposedForm
from zeam.form.silva.utils import convert_request_form_to_unicode
from zeam.form.ztk import validation


REST_ACTIONS_TO_TOKEN = [
    (interfaces.IRESTCloseOnSuccessAction, 'close_on_success'),
    (interfaces.IRESTCloseAction, 'close'),
    (interfaces.IAction, 'send')]


class RESTValidatorForm(SilvaFormData, rest.REST):
    grok.name('form-validate')
    grok.context(IFormCanvas)

    def __translate(self, message):
        return translate(
            message, target_language=self.i18nLanguage, context=self.request)

    def validateField(self, name, value):
        # Inject the value in form with the correct name and decode it.
        self.request.form[name] = value
        convert_request_form_to_unicode(self.request.form)

        info = {'success': True}

        # Look for extractor, extract and validate value.
        for field in self.context.fields:
            extractor = getWidgetExtractor(field, self.context, self.request)
            if extractor is not None:
                if extractor.identifier == name:
                    value, error = extractor.extract()
                    if error is None:
                        error = field.validate(value, self.context.context)
                    if error is not None:
                        info['success'] = False
                        info['error'] = self.__translate(error)
                    break
        return self.json_response(info)

    def GET(self, name, value):
        return self.validateField(name, value)

    def POST(self, name, value):
        return self.validateField(name, value)


class RESTRefreshForm(rest.REST):
    grok.name('form-refresh')
    grok.context(SMIComposedForm)

    def processForm(self, identifier):
        form = self.context.getSubForm(identifier)
        if form is None:
            self.response.setStatus(404)
            return ''
        action, status = form.updateActions()
        form.updateWidgets()
        return self.json_response(
            {'form': form.render(),
             'success': status == SUCCESS})

    def GET(self, identifier):
        convert_request_form_to_unicode(self.request.form)
        return self.processForm(identifier)

    def POST(self, identifier):
        convert_request_form_to_unicode(self.request.form)
        return self.processForm(identifier)


class RESTPopupForm(SilvaFormData, rest.REST, FormCanvas):
    grok.baseclass()

    def __init__(self, context, request):
        # Our custom widgets are registered on ISMILayer. Apply it as well
        alsoProvides(request, ISMILayer)
        # XXX I loved super
        rest.REST.__init__(self, context, request)
        FormCanvas.__init__(self, context, request)

    def __translate(self, message):
        return translate(
            message, target_language=self.i18nLanguage, context=self.request)

    def renderActions(self):
        def renderAction(action):
            for rest_action, action_type in REST_ACTIONS_TO_TOKEN:
                if rest_action.providedBy(action.component):
                    break
            return {'label': self.__translate(action.title),
                    'name': action.identifier,
                    'action': action_type}
        return map(renderAction, self.actionWidgets)

    def processForm(self):
        action, status = self.updateActions()
        self.updateWidgets()
        info = {}
        info['success'] = status == SUCCESS
        if interfaces.IRESTRefreshAction.providedBy(action):
            info['refresh'] = action.refresh
        success_only = interfaces.IRESTSuccessAction.providedBy(action)
        if not (success_only and status == SUCCESS):
            actions = self.renderActions()
            info.update(
                {'label': self.__translate(self.label),
                 'widgets': self.render(),
                 'actions': actions,
                 'default_action': actions[0]['name'] if actions else None})
        return self.json_response(info)

    def GET(self):
        convert_request_form_to_unicode(self.request.form)
        return self.processForm()

    def POST(self):
        convert_request_form_to_unicode(self.request.form)
        return self.processForm()


class RESTFormTemplate(pt.PageTemplate):
    pt.view(RESTPopupForm)


class RESTKupuEditProperties(RESTPopupForm):
    grok.baseclass()
    grok.name('kupu-properties')

    ignoreContent = False
    dataManager = SilvaDataManager
    dataValidators = [validation.InvariantsValidation]

    def setContentData(self, content):
        original_content = content
        if IVersionedContent.providedBy(original_content):
            content = original_content.get_editable()
            if content is None:
                content = original_content.get_previewable()
        super(RESTKupuEditProperties, self).setContentData(content)


class RESTKupuTemplate(pt.PageTemplate):
    pt.view(RESTKupuEditProperties)



# Copyright (c) 2010-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from five import grok
from five.grok.meta import ViewSecurityGrokker
from grokcore.security.components import Permission
from martian.error import GrokError
from zeam.form.silva import form
from zope.component import provideUtility
from zope.security.interfaces import IPermission
import martian


class SilvaAddFormSecurityGrokker(martian.ClassGrokker):
    """Set the correct permission on add forms.
    """
    martian.component(form.SMIAddForm)
    martian.directive(grok.name)
    martian.directive(grok.require)
    martian.priority(800)       # Priority > SilvaFormSecurityGrokker

    def execute(self, factory, config, name, require, **kw):
        if not name:
            raise GrokError(u"Add forms must have a name, the meta_type")

        # We define a permission for the add form and use it. For the
        # time being, there is no way to make it optional.

        permission = Permission(
            unicode('silva.add.' + name.replace(' ', '')),
            unicode('Add %ss' % name))

        # Order is set to -1 to be done as soon as possible.
        config.action(
            discriminator=('utility', IPermission, permission.id),
            callable=provideUtility,
            args=(permission, IPermission, permission.id),
            order=-1)

        grok.require.set(factory, [permission.id])
        return True


class SilvaFormSecurityGrokker(ViewSecurityGrokker):
    """We want to set Zope 2 security on Forms
    """
    martian.component(form.SilvaForm)
    martian.priority(400)


class ZopeFormSecurityGrokker(ViewSecurityGrokker):
    """We want to set Zope 2 security on Forms
    """
    martian.component(form.ZopeForm)



# Copyright (c) 2010-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from Acquisition import aq_inner, aq_parent

from five import grok
from zope.interface import Interface

from silva.core.interfaces import IRoot
from silva.translations import translate as _

from zope.traversing.browser import absoluteURL
from zeam.form.base import SUCCESS, FAILURE
from zeam.form.base.actions import Action, DecoratedAction
from zeam.form.base.interfaces import IFormData
from zeam.form.base.widgets import ActionWidget
from zeam.form.ztk.actions import EditAction as BaseEditAction

from zeam.form.silva import interfaces


class EditAction(BaseEditAction):
    """Edit action
    """
    grok.implements(interfaces.IRESTCloseOnSuccessAction)
    title = _(u"save changes")
    description = _(u"save modifications: alt-s")
    accesskey = u's'

    def __call__(self, form):
        status = super(EditAction, self).__call__(form)
        if interfaces.ISilvaFormData.providedBy(form):
            if status is FAILURE:
                for error in form.formErrors:
                    form.send_message(error.title, type="error")
            else:
                form.send_message(_(u"Changes saved."), type="feedback")
        return status


class PopupAction(Action):
    """An action that opens a popup form.
    """
    action = None

    def __call__(self, form):
        # It should never be called actually.
        return SUCCESS


class PopupWidget(ActionWidget):
    """Widget to style popup buttons
    """
    grok.adapts(PopupAction, IFormData, Interface)

    def url(self):
        return '/'.join(
            (absoluteURL(self.form.context, self.request),
             '++rest++' + self.component.action))


class RemoverWidget(ActionWidget):
    """Widget in red.
    """
    grok.adapts(interfaces.IRemoverAction, IFormData, Interface)

    def htmlClass(self):
        return 'action remover'


class CancelAction(Action):
    """A action to cancel
    """
    grok.implements(interfaces.IRESTCloseAction, interfaces.ICancelerAction)
    title = _(u"cancel")
    description = _(u"go back to the folder view: alt-c")
    accesskey = u'c'

    def __call__(self, form):
        form.redirect(form.url(name="edit"))
        return SUCCESS


class CancelAddAction(CancelAction):
    """Cancel an add action.
    """
    description = _(
        u"go back to the folder view without adding the content: alt-c")

    def __call__(self, form):
        form.redirect(form.url(name="edit"))
        return SUCCESS


class CancelEditAction(CancelAction):
    """Cancel an edit action.
    """
    description = _(
        u"go back to the folder view without editing the content: alt-c")

    def __call__(self, form):
        content = form.context
        if not IRoot.providedBy(content):
            content = aq_parent(aq_inner(content))
        form.redirect(form.url(obj=content, name='edit'))
        return SUCCESS


class CancelWidget(ActionWidget):
    """Widget to style Cancel buttons
    """
    grok.adapts(interfaces.ICancelerAction, IFormData, Interface)

    def htmlClass(self):
        return 'canceler'


class ExtractedDecoratedAction(DecoratedAction):
    """Action that can be used a factory for the decorator @action,
    which extract data itself before calling the decorated method.
    """

    def __call__(self, form):
        data, errors = form.extractData()
        if errors:
            if interfaces.ISilvaFormData.providedBy(form):
                for error in form.formErrors:
                    form.send_message(error.title, type="error")
            return FAILURE
        # We directly give data.
        return super(ExtractedDecoratedAction, self).__call__(form, data)



# -*- coding: utf-8 -*-
# Copyright (c) 2002-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from zope import interface
from zeam.form.base.interfaces import IAction


# Integrate a form in Silva.
class ISilvaFormData(interface.Interface):
    """A form data in Silva.
    """

# Define style for actions
class IRemoverAction(IAction):
    """An action that appear in red.
    """

class ICancelerAction(IAction):
    """An action that cancel something.
    """

# Define categories of actions for RESTForm.
class IRESTAction(IAction):
    """Action design to be used with a RESTForm, that have a special
    meaning.
    """

class IRESTSuccessAction(IRESTAction):
    """Action just send success on success.
    """

class IRESTCloseAction(IRESTAction):
    """Do not call the action callback, just close the RESTForm.
    """

class IRESTCloseOnSuccessAction(IRESTCloseAction, IRESTSuccessAction):
    """Call the action callback, and close the rest Form upon success.
    """

class IRESTRefreshAction(IRESTAction):
    """Trigger the refresh of a form component on the main view.
    """
    refresh = interface.Attribute(u'Identifier to refresh')



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from zope.i18n.interfaces import IUserPreferredLanguages
from zope.i18n.locales import locales, LoadLocaleError


def find_locale(request):
    envadapter = IUserPreferredLanguages(request, None)
    if envadapter is None:
        return None

    langs = envadapter.getPreferredLanguages()
    for httplang in langs:
        parts = (httplang.split('-') + [None, None])[:3]
        try:
            return locales.getLocale(*parts)
        except LoadLocaleError:
            # Just try the next combination
            pass
    else:
        # No combination gave us an existing locale, so use the default,
        # which is guaranteed to exist
        return locales.getLocale(None, None, None)


def decode_to_unicode(string):
    if not isinstance(string, str):
        return string
    try:
        return string.decode('utf-8')
    except UnicodeDecodeError:
        # Log
        return string


def convert_request_form_to_unicode(form):
    for key, value in form.iteritems():
        if isinstance(value, list):
            form[key] = [decode_to_unicode(i) for i in value]
        else:
            form[key] = decode_to_unicode(value)



from setuptools import setup, find_packages
import os

version = '1.3.1'


setup(name='zeam.form.silva',
      version=version,
      description="Integration of the Grok-based Zeam Form system into Silva",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),

      classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='grok form framework silva',
      author='Sylvain Viollon',
      author_email='thefunny@gmail.com',
      url='https://github.com/silvacms/zeam.form.silva',
      license='BSD',
      package_dir={'': 'src'},
      packages=find_packages('src'),
      namespace_packages=['zeam', 'zeam.form'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'five.grok',
        'infrae.layout',
        'martian',
        'megrok.pagetemplate',
        'setuptools',
        'silva.core.conf',
        'silva.core.interfaces',
        'silva.core.messages',
        'silva.core.smi',
        'silva.translations',
        'zeam.form.base',
        'zeam.form.composed',
        'zeam.form.table',
        'zeam.form.viewlet',
        'zeam.form.ztk',
        'zope.component',
        'zope.i18n',
        'zope.interface',
        'zope.schema',
        ],
      entry_points="""
      # -*- Entry points: -*-
      [zeam.form.components]
      file = zeam.form.silva.widgets:register
      """,
      )



# Copyright (c) 2010-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from Acquisition import aq_base
from zExceptions import Redirect

from five import grok
from zope import interface, schema
from megrok import pagetemplate as pt

from zeam.form import base, composed, table, viewlet
from zeam.form.base.datamanager import BaseDataManager
from zeam.form.base.fields import Fields
from zeam.form.base.widgets import Widgets
from zeam.form.base.markers import DISPLAY, SUCCESS, FAILURE, NO_VALUE
from zeam.form.ztk import validation

from zeam.form.silva.interfaces import ISilvaFormData
from zeam.form.silva.utils import find_locale, convert_request_form_to_unicode
from zeam.form.silva.actions import CancelAddAction, CancelEditAction
from zeam.form.silva.actions import EditAction, ExtractedDecoratedAction


from infrae.layout.interfaces import IPage, ILayoutFactory
from grokcore.view.meta.views import default_view_name

from Products.Silva.icon import get_icon_url, get_meta_type_icon_url
from Products.Silva.ExtensionRegistry import extensionRegistry

from zope import component
from zope.cachedescriptors.property import CachedProperty
from zope.configuration.name import resolve
from zope.i18n.interfaces import IUserPreferredLanguages
from zope.publisher.publish import mapply

from silva.core.conf.interfaces import ITitledContent
from silva.core.conf.utils import getFactoryName
from silva.core.interfaces.content import IVersionedContent, IPublishable
from silva.core.layout.interfaces import ISilvaLayer
from silva.core.messages.interfaces import IMessageService
from silva.core.smi.interfaces import IAddingTab, IEditTabIndex
from silva.core.smi.interfaces import ISMILayer
from silva.core.views.views import HTTPHeaderView
from silva.translations import translate as _


class SilvaFormData(object):
    grok.implements(ISilvaFormData)

    @CachedProperty
    def i18nLanguage(self):
        adapter = IUserPreferredLanguages(self.request)
        languages = adapter.getPreferredLanguages()
        if languages:
            return languages[0]
        return 'en'

    def send_message(self, message, type=u""):
        service = component.getUtility(IMessageService)
        service.send(message, self.request, namespace=type)


class ZopeForm(object):
    """Simple Zope Form.
    """
    grok.baseclass()
    dataValidators = [validation.InvariantsValidation]

    @CachedProperty
    def i18nLanguage(self):
        adapter = IUserPreferredLanguages(self.request)
        languages = adapter.getPreferredLanguages()
        if languages:
            return languages[0]
        return 'en'

    def __init__(self, context, request):
        super(ZopeForm, self).__init__(context, request)
        self.__name__ = self.__view_name__

    def __call__(self):
        if not hasattr(self.request, 'locale'):
            # This is not pretty, but no choice.
            self.request.locale = find_locale(self.request)
        convert_request_form_to_unicode(self.request.form)

        return super(ZopeForm, self).__call__()


class SilvaForm(HTTPHeaderView, SilvaFormData):
    """Form in Silva.
    """
    grok.baseclass()
    grok.implements(IPage)
    dataValidators = [validation.InvariantsValidation]

    def __init__(self, context, request):
        super(SilvaForm, self).__init__(context, request)
        self.__name__ = self.__view_name__
        self.layout = None

    @property
    def tab_name(self):
        # XXX should only be available for SMI
        return grok.name.bind().get(self, default=default_view_name)

    def default_namespace(self):
        namespace = super(SilvaForm, self).default_namespace()
        namespace['layout'] = self.layout
        return namespace

    def content(self):
        return self.render()

    def __call__(self):
        self.setHTTPHeaders()
        if not hasattr(self.request, 'locale'):
            # This is not pretty, but no choice.
            self.request.locale = find_locale(self.request)
        convert_request_form_to_unicode(self.request.form)

        layout_factory = component.getMultiAdapter(
            (self.request, self.context,), ILayoutFactory)
        self.layout = layout_factory(self)

        mapply(self.update, (), self.request)
        if self.request.response.getStatus() in (302, 303):
            # A redirect was triggered somewhere in update(). Don't
            # continue processing the form
            return

        self.updateForm()
        if self.request.response.getStatus() in (302, 303):
            return

        return self.layout(self)


class SilvaDataManager(BaseDataManager):
    """Try to use in priority set_ and get_ methods when setting and
    getting values on an object, paying attention to the Acquisition.
    """

    def get(self, identifier):
        if hasattr(aq_base(self.content), 'get_%s' % identifier):
            getter = getattr(self.content, 'get_%s' % identifier)
            return getter()
        if not hasattr(aq_base(self.content), identifier):
            raise KeyError(identifier)
        return getattr(self.content, identifier)

    def set(self, identifier, value):
        if hasattr(aq_base(self.content), 'set_%s' % identifier):
            setter = getattr(self.content, 'set_%s' % identifier)
            return setter(value)
        return setattr(self.content, identifier, value)


class ZMIForm(ZopeForm, base.Form):
    """Regular ZMI forms.
    """
    grok.baseclass()
    grok.require('zope2.ViewManagementScreens')


class ZMIFormTemplate(pt.PageTemplate):
    pt.view(ZMIForm)


class ZMIComposedForm(ZopeForm, composed.ComposedForm):
    """ZMI Composed forms.
    """
    grok.baseclass()
    grok.require('zope2.ViewManagementScreens')


class ZMIComposedFormTemplate(pt.PageTemplate):
    pt.view(ZMIComposedForm)


class ZMISubForm(composed.SubForm):
    """ZMI Composed forms.
    """
    grok.baseclass()


class ZMISubFormTemplate(pt.PageTemplate):
    pt.view(ZMISubForm)


class SMIForm(SilvaForm, base.Form):
    """Regular SMI form.
    """
    grok.baseclass()
    grok.layer(ISMILayer)
    grok.require('silva.ChangeSilvaContent')

    @property
    def icon_url(self):
        return get_icon_url(self.context, self.request)


class SMIFormTemplate(pt.PageTemplate):
    pt.view(SMIForm)


class PublicForm(SilvaForm, base.Form):
    """Regular SMI form.
    """
    grok.baseclass()
    grok.layer(ISilvaLayer)
    grok.require('zope.Public')


class SMIComposedForm(SilvaForm, composed.ComposedForm):
    """SMI Composed forms.
    """
    grok.baseclass()
    grok.layer(ISMILayer)
    grok.require('silva.ChangeSilvaContent')

    @property
    def icon_url(self):
        return get_icon_url(self.context, self.request)


class SMIComposedFormTemplate(pt.PageTemplate):
    pt.view(SMIComposedForm)


class SMISubForm(SilvaFormData, composed.SubForm):
    """SMI Sub forms.
    """
    grok.baseclass()


class SMISubFormTemplate(pt.PageTemplate):
    pt.view(SMISubForm)


class SMISubFormGroup(SilvaFormData, composed.SubFormGroup):
    """SMI sub form group.
    """
    grok.baseclass()


class SMISubFormGroupTemplate(pt.PageTemplate):
    pt.view(SMISubFormGroup)


class SMISubTableForm(SilvaFormData, table.SubTableForm):
    """SMI Sub table forms.
    """
    grok.baseclass()


class SMISubTableFormTemplate(pt.PageTemplate):
    pt.view(SMISubTableForm)


class IAddOptions(interface.Interface):
    position = schema.Int(
        title=_(u"add position"),
        description=_(
            u"Position in the container to which the content will be added."),
        default=0,
        min=0)


def add_position_available(form):
    if  'addform.options.position' not in form.request.form:
        return False
    addable = extensionRegistry.get_addable(form.__name__)
    return IPublishable.implementedBy(addable['instance'])


class SMIAddForm(SMIForm):
    """ SMI add form
    """
    grok.baseclass()
    grok.implements(IAddingTab)

    prefix = 'addform'
    tab = 'edit'

    fields = Fields(ITitledContent)
    dataManager = SilvaDataManager
    ignoreContent = True
    actions = base.Actions()
    optionFields = Fields(IAddOptions)
    optionFields['position'].prefix = 'options'
    optionFields['position'].mode = 'readonly'
    optionFields['position'].available = add_position_available

    def updateWidgets(self):
        super(SMIAddForm, self).updateWidgets()
        optionWidgets = Widgets(form=self, request=self.request)
        optionWidgets.extend(self.optionFields)
        optionWidgets.update()
        self.fieldWidgets.extend(optionWidgets)

    @property
    def tab_name(self):
        return '+/' + self.__name__

    @property
    def icon_url(self):
        return get_meta_type_icon_url(self.__name__, self.request)

    def _add(self, parent, data):
        """Purely create the object. This method can be overriden to
        support custom creation needs.
        """
        # Search for an addable and a factory
        addable = extensionRegistry.get_addable(self.__name__)
        if not addable:
            raise ValueError(u"Content factory cannot be found. ")

        factory = getattr(
            resolve(addable['instance'].__module__),
            getFactoryName(addable['instance']))

        # Build the content
        identifier = str(data.getWithDefault('id'))
        factory(parent, identifier, data.getWithDefault('title'))
        content = getattr(parent, identifier)

        self._edit(parent, content, data)
        return content

    def _edit(self, parent, content, data):
        # Set from value
        editable_content = self.dataManager(content.get_editable())
        for key, value in data.iteritems():
            if key not in ITitledContent and value is not NO_VALUE:
                editable_content.set(key, value)

    def _move(self, parent, content):
        data, errors = self.extractData(self.optionFields)
        position = data.getWithDefault('position')
        if position > 0:
            # XXX bug in Folder to have move_to(2) you actually need 1.
            parent.move_to([content.getId()], position - 1)

    @base.action(
        _(u'save + edit'),
        identifier='save_edit',
        description=_(
            u"create the content and go on its edit view: alt-e"),
        accesskey=u'e',
        factory=ExtractedDecoratedAction)
    def save_edit(self, data):
        try:
            content = self._add(self.context, data)
            self._move(self.context, content)
        except ValueError, error:
            self.send_message(error, type="error")
            return FAILURE
        self.send_message(
            _(u'Added ${meta_type}.', mapping={'meta_type': self.__name__}),
            type="feedback")
        self.redirect(self.url(content, name="edit"))
        return SUCCESS

    @base.action(
        _(u'save'),
        description=_(
            u"create the content and go back to the folder view: alt-s"),
        accesskey=u's',
        factory=ExtractedDecoratedAction)
    def save(self, data):
        try:
            content = self._add(self.context, data)
            self._move(self.context, content)
        except ValueError, error:
            self.send_message(error.args[0], type=u"error")
            return FAILURE
        self.send_message(
            _(u'Added ${meta_type}.', mapping={'meta_type': self.__name__}),
            type="feedback")
        self.redirect(self.url(name="edit"))
        return SUCCESS

    actions.append(CancelAddAction())


class SMIAddFormTemplate(pt.PageTemplate):
    pt.view(SMIAddForm)


class SMIEditForm(SMIForm):
    """SMI Edit form.
    """
    grok.baseclass()
    grok.name('tab_edit')
    grok.implements(IEditTabIndex)

    prefix = 'editform'
    tab = 'edit'

    dataManager = SilvaDataManager
    ignoreContent = False
    actions = base.Actions(
        EditAction(),
        CancelEditAction())

    def update(self):
        """ If we have a versioned content and it has a published/approved
        version, the we set the form in display mode.
        """
        super(SMIEditForm, self).update()
        if IVersionedContent.providedBy(self.context):
            if ((not self.context.get_editable()) or
                self.context.is_version_approval_requested()):
                self.mode = DISPLAY
                self.actions = base.Actions()

    def setContentData(self, content):
        original_content = content
        if IVersionedContent.providedBy(original_content):
            content = original_content.get_editable()
            if content is None:
                content = original_content.get_previewable()
        super(SMIEditForm, self).setContentData(content)


class SMIViewletForm(SilvaFormData, viewlet.ViewletForm):
    """ Base form in viewlet
    """
    grok.baseclass()

    def available(self):
        for action in self.actions:
            if action.available(self):
                return True
        return False

    def update(self):
        if not hasattr(self.request, 'locale'):
            self.request.locale = find_locale(self.request)
        convert_request_form_to_unicode(self.request.form)
        return super(SMIViewletForm, self).update()

    def redirect(self, url):
        # Raise redirect exception to be not to render the current
        # page anymore.
        raise Redirect(url)

PublicViewletForm = SMIViewletForm


class SMIContentProviderForm(SilvaFormData, viewlet.ViewletManagerForm):
    grok.baseclass()

    def update(self):
        if not hasattr(self.request, 'locale'):
            self.request.locale = find_locale(self.request)
        convert_request_form_to_unicode(self.request.form)
        return super(SMIContentProviderForm, self).update()

    def redirect(self, url):
        # Raise redirect exception to be not to render the current
        # page anymore.
        raise Redirect(url)

PublicContentProviderForm = SMIContentProviderForm
